/*
 *  Copyright 2008 CNRS
 *  Author: Florent Lamiraux
 */

#ifndef HPPWALKFOOTPLANNER_H
#define HPPWALKFOOTPLANNER_H

#include "hppModel/hppDevice.h"
#include "hppCore/hppPlanner.h"
#include "hppWalkFootPlanner/hppWalkStepSequence.h"
#include "kwsPlusDirectPath.h"

KIT_PREDEF_CLASS(CkppKCDBox);

/**
  \brief Collision-free foot step planning for humanoid robot.

  This class implements collision free foot step planning for a humanoid robot. 
The robot is represented by a bounding box moving in the plane. Path planning is performed 
in two steps:
\li first a collision free path is built for the bounding box, the default steering method is ChppWalkSteeringMethod,
\li then sequences of step sequences (ChppWalkStepSequence) are computed along the path of the box.

   <b> How to use this class</b>

   To use this class, 
   \li create an object ChppWalkFootPlanner, by specifying the step size,
   \li create a robot bounding box with appropriate size (ChppWalkFootPlanner::createRobotBox),
   \li insert this robot in the object (ChppPlanner::addHppProblem),
   \li call ChppWalkFootPlanner::initializeProblem to set the default steering method and distance function,
   \li add obstacles using ChppPlanner::addObstacle,
   \li choose initial and goal configurations of the box using ChppWalkFootPlanner::initConfig and ChppWalkFootPlanner::goalConfig and 
   \li call ChppWalkFootPlanner::solve to compute a step sequences,
   \li the result can be accessed through ChppWalkFootPlanner::sequenceOfStepSequences.

 */

class ChppWalkFootPlanner : public ChppPlanner {
public:
  /**
     \brief Constructor

     \param inStepLength Abscissa of the landing foot in the frame of the support foot in straight line motions.
     \param inStepSpace Absolute value of the ordinate of the landing foot in the frame of the support foot
     \param inSecurityDistance Distance above which centers of feet should remain in two consecutive steps.
  */
  ChppWalkFootPlanner(double inStepLength, double inStepSpace, double inSecurityDistance);

  virtual ~ChppWalkFootPlanner();

  /**
     \name Read access to parameters

     @{
  */

  /**
     \brief Get step length
  */
  double stepLength() const {return attStepLength;};

  /**
     \brief Get lateral step spacing
  */
  double stepSpacing() const {return attStepSpacing;};

  /**
     \brief Get security distance between foot position on the ground.
  */
  double securityDistance() const { return attSecurityDistance;};

  /**
     @}
  */

  /**
     \name Problem resolution
     @{
  */

  /**
     \brief Set initial configuration of Walk Planning problem
     \param x position and orientation of the robot.
     \param y position and orientation of the robot.
     \param theta position and orientation of the robot.
  */
  virtual ktStatus initConfig(double x, double y, double theta);
  
  /**
     \brief Set goal configuration of Walk Planning problem
     \param x position and orientation of the robot.
     \param y position and orientation of the robot.
     \param theta position and orientation of the robot.
  */
  virtual ktStatus goalConfig(double x, double y, double theta);

  /**
     \brief Create a robot the body of which is a box.

    \param xMin, xMax, yMin, yMax, zMin, zMax coordinates of the box vertices in joint frame.

     The root joint (and only one) is a freeflyer.
     An extra dof is added to the robot in order to use CflicSteeringMethod.
     
  */
  ChppDeviceShPtr createRobotBox(double xMin, double yMin, double zMin,
				 double xMax, double yMax, double zMax); // const;

  /*
    \brief Reset size of bounding box

    \param inXmin, inXmax, inYmin, inYmax, inZmin, inZmax coordinates of the box vertices in joint frame.
  */
  bool setBoundingBoxSize(double inXmin, double inXmax, double inYmin, 
			  double inYmax, double inZmin, double inZmax);

  /**
     \brief Initialize a Walk Planning problem.
     Compute the bounding box of the first robot in problem  vector (in given configuration) and create a robot with this box. Then insert this new robot at beginning of problem vector.
     
     The box robot is placed in the first problem by a push_front().

  */
  virtual ktStatus initializeProblem();
  
  /**
     @}
  */

  /**
     \name Problem resolution
     @{
  */

  /** 
    \brief Set roadmap builder of i-th problem.

    \param inRank Rank of problem in ChppPlanner::hppProblemVector.
    \param inRoadmapBuilder roadmap builder.
    \param inDisplay whether the roadmap should be displayed in the interface.

    Call ChppPlanner::roadmapBuilderIthProblem and set ChppWalkPlannerDelegate 
    to roadmap builder if rank=0 (Robot bounding box).
  */
  ktStatus roadmapBuilderIthProblem(unsigned int inRank, CkwsRoadmapBuilderShPtr inRoadmapBuilder,
				    bool inDisplay=false);

  /**
     \brief Iteratively solve a walk motion planning problem for a humanoid robot.
     \return KD_OK or KD_ERROR
  */
  virtual ktStatus solve();

  /**
     \brief Compute the step sequence from the latest path of the box.
  */
  ktStatus computeStepSequenceFromBoxPath();

  /**
     \brief get the relativeFootPrint stack that follow a path
     \return attRelativeFootPrintVector

     The path of the bounding box is split into sections of discontinuous velocity.
     A sequence of step is computed for each section. The robot is thus supposed to 
     stop between two successive sequences.
  */
  const std::vector<ChppWalkStepSequence>& sequenceOfStepSequences() const {
    return attSequenceOfStepSequences;
  };

  /** 
   *@}
   */

private:

  /**
     \brief  compute from a box's path all the print foot for HRP2 : this is stored in m_RelativeFootPositions vector
     \param inBoxPath Path of the bounding box
     \retval outStepSequence sequence of steps
  */
  void computeAStepSequenceFromAPath(CkwsPathShPtr inBoxPath, ChppWalkStepSequence& outStepSequence) ;
 
  /**
     \brief Split a path of the bounding box into sub-paths of continuous velocity

     \param inBoundingBoxPath Path of the bounding box
     \retval outPathVector vector of paths of continuous velocity
     \return KD_OK | KD_ERROR
  */
  ktStatus splitPathByDiscontinuousVelocity(CkwsPathShPtr inBoundingBoxPath, std::vector<CkwsPathShPtr>& outPathVector);

  /**
     \brief Whether path should be split between two consecutive direct paths
     \param inDP1, inDP2 consecutive direct paths

     If the velocity is discontinuous, the step sequence should stop at the end of inDP1
     and a new one should start at the beginning of inDP2.
  */
  bool shouldSplitBetween(const CkwsPlusDirectPathConstShPtr& inDP1, 
			  const CkwsPlusDirectPathConstShPtr& inDP2) const;
  

  /**
     \brief Set the user weight scaler of Rz degree of freedom

     This parameter determines the length parameterization of CkwsDPLinear.
  */
  void setUserWeightScaler(CkppJointComponentShPtr inKppJoint);

  /**
     \brief Get the robot bounding box
  */
  CkppKCDBoxShPtr getRobotBoundingBox();

  /**
     \brief Step sequence of relative foot use to compute the path

     Previously ChppWalkPatGen::attRelativeFootPrintVector
  */
  std::vector<ChppWalkStepSequence> attSequenceOfStepSequences;

  /**
     \brief Length of the steps 
     Abscissa of the landing foot in the frame of the support foot in straight line motions.
  */
  double attStepLength; 
  
  /**
     \brief Space between feet
     Absolute value of the ordinate of the landing foot in the frame of the support foot
  */
  double attStepSpacing;
  
  /**
     \brief Security distance between feet
     Distance above which centers of feet should remain in two consecutive steps.
  */
  double attSecurityDistance;
};

#endif
