/*
 *  Copyright 2008 CNRS
 *  Author: Florent Lamiraux
 */

#include <iostream>

#include "hppModel/hppBody.h"
#include "hppWalkFootPlanner/hppWalkFootPlanner.h" 

#include "jrlMathTools/angle.h"
#include "kwsPlusSMLinear.h"

#include "hppWalkFootPlanner/hppWalkPlannerDelegate.h"
#include "hppWalkFootPlanner/hppWalkSteeringMethod.h"
#include "hppWalkFootPlanner/hppWalkDistance.h"

#include "KineoWorks2/kwsJointFreeFlyer.h"
#include "KineoWorks2/kwsJointDof.h"
#include "KineoModel/kppDeviceComponent.h"
#include "KineoModel/kppFreeFlyerJointComponent.h"
#include "KineoModel/kppJointComponent.h"
#include "KineoModel/kppSolidComponentRef.h"
#include "KineoModel/kppExtraDofComponent.h"
#include "KineoKCDModel/kppKCDBox.h"

#if DEBUG==2
#include "kwsIO/kwsioMat.h"
#endif

// Select verbosity at configuration by setting CXXFLAGS="... -DDEBUG=[1 or 2]"
#if DEBUG==3
#define ODEBUG3(x) std::cout << "ChppWalkPlanner:" << x << std::endl
#define ODEBUG2(x) std::cerr << "hppWalkFootPlanner:" << x << std::endl
#define ODEBUG1(x) std::cerr << "hppWalkFootPlanner:" << x << std::endl
#elif DEBUG==2
#define ODEBUG3(x)
#define ODEBUG2(x) std::cerr << "hppWalkFootPlanner:" << x << std::endl
#define ODEBUG1(x) std::cerr << "hppWalkFootPlanner:" << x << std::endl
#elif DEBUG==1
#define ODEBUG3(x)
#define ODEBUG2(x)
#define ODEBUG1(x) std::cerr << "hppWalkFootPlanner:" << x << std::endl
#else
#define ODEBUG3(x)
#define ODEBUG2(x)
#define ODEBUG1(x)
#endif

#define TOR_OBSTACLE 0.015

ChppWalkFootPlanner::ChppWalkFootPlanner(double inStepLength, double inStepSpace, 
					 double inSecurityDistance) :
  ChppPlanner(), attStepLength(inStepLength), attStepSpacing(inStepSpace), 
  attSecurityDistance(inSecurityDistance)
{
}

ChppWalkFootPlanner::~ChppWalkFootPlanner()
{
}

// ==========================================================================

ktStatus ChppWalkFootPlanner::initConfig(double inX, double inY, double inTheta)
{

  if (getNbHppProblems() < 1) {
    ODEBUG1(":initConfig: no robot defined : ");
    return KD_ERROR;
  }

  /*
    Build configuration for bounding box.
  */

  const CkppDeviceComponentShPtr boundingBox = robotIthProblem(0);
 
  if( !boundingBox){
    ODEBUG1(":initConfig: the bounding box is empty");
  }
  
  std::vector<double> dofValue;
  dofValue.push_back(0.0);
  dofValue.push_back(inX);
  dofValue.push_back(inY);
  dofValue.push_back(0.0);
  dofValue.push_back(0.0);
  dofValue.push_back(0.0);
  dofValue.push_back(inTheta);
  
  CkwsConfigShPtr initConfBox =  CkwsConfig::create(boundingBox, dofValue);

  if (!initConfBox) {
    ODEBUG1(":initConfig: could not createthe init  configuration boundingBox.");
    return KD_ERROR;
  }
    
  if (initConfIthProblem(0, initConfBox) == KD_ERROR) {
    ODEBUG1(":initConfig could not set initial configuration of robot box.");
    return KD_ERROR;
  }

  return KD_OK;
}

// ==========================================================================

ktStatus ChppWalkFootPlanner::goalConfig(double inX, double inY, double inTheta)
{
  if (getNbHppProblems() < 1) {
    ODEBUG1(":goalConfig - not enought robot has been defined : " << getNbHppProblems());
    return KD_ERROR;
  }

  /*
    Build configuration for bounding box.
  */  

  const CkppDeviceComponentShPtr boundingBox = robotIthProblem(0);
  
  if( !boundingBox){
    ODEBUG1(":goalConfig: the bounding box is empty") ;
  }

  std::vector<double> dofValue;
  dofValue.push_back(0.0);
  dofValue.push_back(inX);
  dofValue.push_back(inY);
  dofValue.push_back(0.0);
  dofValue.push_back(0.0);
  dofValue.push_back(0.0);
  dofValue.push_back(inTheta);
  
  CkwsConfigShPtr goalConfBox =  CkwsConfig::create(boundingBox, dofValue);
  
  if (!goalConfBox) {
    ODEBUG1(":goalConfig : could not create the goal  configuration boundingBox.");
    return KD_ERROR;
  }
    
  if (goalConfIthProblem(0, goalConfBox) == KD_ERROR) {
    ODEBUG1(":goalConfig :could not set initial configuration of robot box.");
    return KD_ERROR;
  }
 
  return KD_OK;
}

// ==========================================================================

ktStatus ChppWalkFootPlanner::initializeProblem()
{
  unsigned int nbRobots = getNbHppProblems();

  if (nbRobots == 0) {
    ODEBUG1(":initializeProblem: there is no robot.");
    return KD_ERROR;
  }

  CkppDeviceComponentShPtr robotBox = robotIthProblem(0);
  //
  // Set tolerance of obstacles to default values
  //
  std::vector<CkcdObjectShPtr> oList = obstacleList();
  for(unsigned int i =0; i<oList.size(); i++){
    oList[i]->tolerance(TOR_OBSTACLE);
    ODEBUG2("tolerance " << TOR_OBSTACLE << " set to obstacle " << i);
  }

  double distanceThreshold = 2.0;
  ODEBUG2(":initializeProblem: set ChppWalkSteeringMethod");
  ChppWalkSteeringMethodShPtr sm = ChppWalkSteeringMethod::create(distanceThreshold);
  steeringMethodIthProblem(0, sm);
  
  ODEBUG2(":initializeProblem: set ChppWalkDistance");
  ChppWalkDistanceShPtr distance = ChppWalkDistance::create(distanceThreshold);
  robotBox->distance(distance);

  ODEBUG2(":initializeProblem: return true.");
  return KD_OK;
}

// ==========================================================================

ktStatus ChppWalkFootPlanner::roadmapBuilderIthProblem(unsigned int inRank, 
						       CkwsRoadmapBuilderShPtr inRoadmapBuilder,
						       bool inDisplay)
{
  ODEBUG2(" entering method roadmapBuilderIthProblem().");
  if (ChppPlanner::roadmapBuilderIthProblem(inRank, inRoadmapBuilder, 
					    inDisplay) == KD_ERROR) {
    return KD_ERROR;
  }
  if (inRank == 0) {
    ChppWalkPlannerDelegate* delegate = new ChppWalkPlannerDelegate(inRoadmapBuilder);
    CkwsPlusSMLinearShPtr steeringMethod = 
      CkwsPlusSMLinear::create(std::vector<double>(), false);
    
    delegate->steeringMethod(steeringMethod);
    if (inRoadmapBuilder->addDelegate(delegate) == KD_ERROR) {
      return KD_ERROR;
    }
  }
  return KD_OK;
}

// ==========================================================================

ktStatus ChppWalkFootPlanner::solve()
{
  ktStatus status ;
  
  
  ODEBUG2("_______________________ PHASE 1 ____________________________" ) ;
  
 
  if (!robotIthProblem(0)) {
    ODEBUG1(":solve: HRP2 BOX not Loaded") ;
    return KD_ERROR ;
  }

  if(!initConfIthProblem(0)){
    ODEBUG1(":solve: Init configuration not Initialized") ; 
    return KD_ERROR ;
  }

  if(!goalConfIthProblem(0)){
    ODEBUG1(":solve: goal configuration not Initialized") ; 
    return KD_ERROR ;
  }

  status = solveOneProblem(0);
  if (status == KD_ERROR) {
    return status;
  }

  ODEBUG2(":solve:______________ PHASE 2: step sequence from path____" ) ;
  
  status = computeStepSequenceFromBoxPath();
  return status ;
}


// ==========================================================================

ktStatus ChppWalkFootPlanner::computeStepSequenceFromBoxPath()
{
  //some checking
  if(hppProblemVector.size() < 1) {
    ODEBUG1(":computeStepSequenceFromBoxPath() : No Device.") ;
    return KD_ERROR ;
  }

  CkwsPathShPtr pathBox = getPath(0 , getNbPaths(0)-1 );
  if (!pathBox) {
    ODEBUG1(":computeStepSequenceFromBoxPath: no path found for bounding box");
    return KD_ERROR;
  }
  /*
    The path of the bounding box is split into sub-paths of continuous velocities.
    Then, step sequences are computed for each sub-path successively.
  */
  std::vector<CkwsPathShPtr> pathVector;
  if (splitPathByDiscontinuousVelocity(pathBox, pathVector) != KD_OK) {
    return KD_ERROR;
  }
  
  attSequenceOfStepSequences.clear();
  for (unsigned int iPath=0; iPath < pathVector.size(); iPath++) {
    attSequenceOfStepSequences.push_back(ChppWalkStepSequence());
    computeAStepSequenceFromAPath(pathVector[iPath], attSequenceOfStepSequences[iPath]);
  }
  return KD_OK;
}

// ==========================================================================

void ChppWalkFootPlanner::computeAStepSequenceFromAPath(CkwsPathShPtr inBoxPath, 
							ChppWalkStepSequence& outStepSequence)
{

  ChppWalkStepSequence::TfootPrint footPrint ;
  double fromStart = 0.0 ;
  double footPlus = 0.0, distanceBetweenFeet = 0.0 ;
  CkwsConfigShPtr aConfig ;
  double xCourbe, yCourbe;
  double xNewAbsolu, yNewAbsolu;
  double xNewRelatif, yNewRelatif;
  double xOld, yOld;
  double x, y;
  double xInit, yInit;
  jrlMathTools::Cangle thetaNewRelatif, thetaNewAbsolu, thetaCourbe, thetaInit, thetaOld, theta;

  int lr=-1;

  //compute the real length of the path
  double lengthPath = inBoxPath->length() ;
  int nbFullStep = (int) (lengthPath / stepLength()) ;

  // initial position of the box. The pattern generator starts from config (0,0,0).
  // We need to move the box path by the inverse of the initial configuration.
  aConfig = inBoxPath->configAtStart() ;
  xInit = aConfig->dofValue(1) ;    
  yInit = aConfig->dofValue(2);
  thetaInit = aConfig->dofValue(6);

  outStepSequence.setInitialPosition(xInit, yInit, 0, thetaInit);
  /*
    Choose first foot to move
  */
  double l = min(lengthPath, stepLength());
  CkwsConfigShPtr config(inBoxPath->configAtDistance(l));
  x = config->dofValue(1);
  y = config->dofValue(2);

  if (-sin(thetaInit)*(x-xInit) + cos(thetaInit)*(y-yInit) > 0) {
    /* moving to the left*/
    outStepSequence.leftFootFirst(true);
    lr = -1;
  }
  else {
    /* moving to the right*/
    outStepSequence.leftFootFirst(false);
    lr = 1;
  }

  // clean the vector
  outStepSequence.clear() ;

  // first paralelle feet
  xOld = footPrint.sx = 0.0 ;
  yOld = footPrint.sy = lr*stepSpacing()/2 ;
  thetaOld = footPrint.theta = 0.0 ;
  outStepSequence.push_back(footPrint);
  lr = lr * -1 ;

  for ( int count=1 ; count <= nbFullStep+1 ; count++) {

    //distance from the start
    fromStart = count * stepLength() ;

    if (count == (nbFullStep+1)) {
      //for the last step
      aConfig = inBoxPath->configAtEnd();
    } else {
      // for the other
      aConfig = inBoxPath->configAtDistance(fromStart) ;
    }

    x = aConfig->dofValue(1);
    y = aConfig->dofValue(2);
    theta = aConfig->dofValue(6);

    //replace the point in the PatternGenerator Frame
    xCourbe = (x - xInit)*cos(thetaInit) + (y - yInit)*sin(thetaInit);
    yCourbe = -(x - xInit)*sin(thetaInit) + (y - yInit)*cos(thetaInit);
    thetaCourbe = theta-thetaInit;

    // CHECKING - to keep the foot in a security space especialy in curve
    do {
      //compute x, y, theta Absolute of the new foot
      if (lr == 1) {
	// left foot moving
	xNewAbsolu = (stepSpacing()+footPlus)/2 * -sin(thetaCourbe) + xCourbe ;
	yNewAbsolu = (stepSpacing()+footPlus)/2 * cos(thetaCourbe) + yCourbe ;
      } else {
	//right foot moving
	xNewAbsolu = (stepSpacing()+footPlus)/2 *  sin(thetaCourbe) + xCourbe ;
	yNewAbsolu = (stepSpacing()+footPlus)/2 *  -cos(thetaCourbe) + yCourbe ;
      }
      distanceBetweenFeet = sqrt(pow(xNewAbsolu-xOld,2) + pow(yNewAbsolu-yOld,2)) ;
      footPlus = footPlus + 0.01 ;
    } while ( distanceBetweenFeet < securityDistance() ) ;
    thetaNewAbsolu = thetaCourbe ;

    //compute x, y ,theta Relatif of the new feet from the old feet
    // WARNING : express xNewRelatif , yNewRelatif in the Old foot frame with the orientation of the New foot
    // S=[XNewRelatif, yNewRelatif, thetaNewRelatif] ,  p=[xNewAbsolu, yNewAbsolu], pOld=[xOld, yOld], R(dtheta+thetaOld)= rotation matrix
    // R * S = (p - pOld)
    // BECAUSE : WPG first compute the rotation of the foot then the translation.
    xNewRelatif = cos(thetaNewAbsolu)*(xNewAbsolu-xOld) + sin(thetaNewAbsolu)*(yNewAbsolu-yOld) ;
    yNewRelatif = -sin(thetaNewAbsolu)*(xNewAbsolu-xOld) + cos(thetaNewAbsolu)*(yNewAbsolu-yOld) ;
    thetaNewRelatif = thetaNewAbsolu - thetaOld ;
    
    //stock the info in footPrint
    footPrint.sx = xNewRelatif ;
    footPrint.sy = yNewRelatif ;
    footPrint.theta = thetaNewRelatif*180/M_PI ;  // en degree
    outStepSequence.push_back(footPrint);
    lr = lr * -1 ;

    //stock the info in Old
    xOld = xNewAbsolu ;
    yOld = yNewAbsolu ;
    thetaOld = thetaNewAbsolu ;
    footPlus = 0.0 ;

  }

  //finally put feet forward parallele
  footPrint.sx = 0.0 ;
  footPrint.sy = lr*stepSpacing() ;
  footPrint.theta = 0.0 ;  /* en degree */
  outStepSequence.push_back(footPrint);
  lr = lr * -1 ;

}

// ==========================================================================

ktStatus ChppWalkFootPlanner::splitPathByDiscontinuousVelocity(CkwsPathShPtr inBoundingBoxPath, 
							       std::vector<CkwsPathShPtr>& outPathVector)
{
  CkwsPlusDirectPathConstShPtr previousDirectPath;
  for (unsigned int directPathId=0; directPathId<inBoundingBoxPath->countDirectPaths(); 
       directPathId++) {
    bool shouldSplit=false;
    CkwsPlusDirectPathConstShPtr directPath = 
      KIT_DYNAMIC_PTR_CAST(const CkwsPlusDirectPath, 
			   inBoundingBoxPath->directPath(directPathId));
    if (!directPath) {
      ODEBUG1("::splitPathByContinuousVelocity: direct path should be of type CkwsPlusDirectPath");
      return KD_ERROR;
    }
    if (directPathId == 0) {
      shouldSplit = true;
    }
    else {
      if (shouldSplitBetween(previousDirectPath, directPath)) {
	shouldSplit = true;
      }
      else {
	shouldSplit = false;
      }
    }
    if (shouldSplit) {
      CkwsPathShPtr subPath = CkwsPath::createWithDirectPath(directPath);
      outPathVector.push_back(subPath);
    }
    else {
      outPathVector[outPathVector.size()-1]->appendDirectPath(directPath);
    }
    previousDirectPath = directPath;
  }

  return KD_OK;
}

// ==========================================================================

bool ChppWalkFootPlanner::shouldSplitBetween(const CkwsPlusDirectPathConstShPtr& inDP1,
					     const CkwsPlusDirectPathConstShPtr& inDP2) const
{
  CkwsDeviceShPtr device = inDP1->device();
  unsigned int dim = device->countDofs();
  std::vector<double> velocity1(dim);
  std::vector<double> velocity2(dim);

  double l1 = inDP1->length();
  inDP1->getVelocityAtDistance(l1 , velocity1);
  inDP2->getVelocityAtDistance(0, velocity2);

  CkwsJointShPtr rootJoint = KIT_DYNAMIC_PTR_CAST(CkwsJointFreeFlyer, device->rootJoint());
  if (!rootJoint) {
    ODEBUG1(":shoudSplitBetween: root joint of device is not free flyer");
    return true;
  }

  CkwsDofShPtr dof = device->dof(0);
  double maxCurvature = dof->vmax();

  if (maxCurvature <= 0) {
    ODEBUG1(":shouldSplitBetween: bound on curvature is not positive.");
    return true;
  }

  /*
    x
  */
  dof = rootJoint->dof(0);
  double wx = dof->weight();
  unsigned int dofId;
  if (device->getRankOfDof(dof, dofId) == KD_ERROR) {
    ODEBUG1(":shoudSplitBetween: dof x not found");
    return true;
  }
  double dx1 = velocity1[dofId];
  double dx2 = velocity2[dofId];

  /*
    y
  */
  dof = rootJoint->dof(1);
  double wy = dof->weight();
  if (device->getRankOfDof(dof, dofId) == KD_ERROR) {
    ODEBUG1(":shoudSplitBetween: dof y not found");
    return true;
  }
  double dy1 = velocity1[dofId];
  double dy2 = velocity2[dofId];
  /*
    theta
  */
  dof = rootJoint->dof(5);
  double wt = dof->weight();
  if (device->getRankOfDof(dof, dofId) == KD_ERROR) {
    ODEBUG1(":shoudSplitBetween: dof theta not found");
    return true;
  }
  double dt1 = velocity1[dofId];
  double dt2 = velocity2[dofId];

  /*
    Test that direct paths can be followed without stopping
  */
  double cosV1V2 = wx*wx*dx1*dx2+wy*wy*dy1*dy2+wt*wt*dt1*dt2;
  double normV1 = sqrt(wx*wx*dx1*dx1+wy*wy*dy1*dy1+wt*wt*dt1*dt1);
  double normV2 = sqrt(wx*wx*dx2*dx2+wy*wy*dy2*dy2+wt*wt*dt2*dt2);

  double angle = acos(cosV1V2/(normV1*normV2));
  double length = max(l1, stepLength());

  if (angle > length*maxCurvature) {
    return true;
  }
  return false;
}

// ==========================================================================

ChppDeviceShPtr ChppWalkFootPlanner::createRobotBox(double xMin, double yMin, double zMin,
						    double xMax, double yMax, double zMax) // const
{

  ChppDeviceShPtr hppDevice = ChppDevice::create("Humanoid Box");

  CkitMat4 absPos;
  CkppKCDBoxShPtr polyBox = CkppKCDBox::create("H.Box",xMax-xMin, yMax-yMin, zMax-zMin);
  polyBox->diffuseColor( CkppColor( 0.2f, 0.2f, 0.6f, 0.4f ) );
  absPos.translate(.5*(xMin+xMax), .5*(yMin+yMax), .5*(zMin+zMax));
  polyBox->setAbsolutePosition(absPos);
  polyBox->makeCollisionItem();
  ODEBUG3(":createRobotBox: input parameters");
  ODEBUG3(":createRobotBox: xMin = " << xMin);
  ODEBUG3(":createRobotBox: xMax = " << xMax);
  ODEBUG3(":createRobotBox: yMin = " << yMin);
  ODEBUG3(":createRobotBox: yMax = " << yMax);
  ODEBUG3(":createRobotBox: zMin = " << zMin);
  ODEBUG3(":createRobotBox: zMax = " << zMax);

  ODEBUG3(":createRobotBox: ");

  ODEBUG3(":createRobotBox: Size of the box ");
  ODEBUG3(":createRobotBox: lx = " << xMax-xMin << ", ly = " << yMax-yMin 
	  << ", lz = " << zMax-zMin);
  ODEBUG3(":createRobotBox: Position of the box");
  ODEBUG3(":createRobotBox: x = " << .5*(xMin+xMax) << ", y = " << .5*(yMin+yMax)
	  << ", z = " << .5*(zMin+zMax));
  
  // create ExtraDof
  CkppExtraDofComponentShPtr kwsExtraDof ;
  kwsExtraDof = CkppExtraDofComponent::create(false, "Curvature");
  // Bound curvature
  kwsExtraDof->CkppDofComponent::bounds(-1, 1);
  kwsExtraDof->CkppDofComponent::isBounded(true);
  kwsExtraDof->CkppDofComponent::isLocked(false) ;
  hppDevice->addExtraDof(kwsExtraDof);
  
  
  // build plan root joint
  CkppJointComponentShPtr kppJoint = CkppFreeFlyerJointComponent::create( "Rootjoint.H.Box" );
  // Lock non-planar dofs
  // Z
  kppJoint->kwsJoint()->dof(2)->isLocked(true);
  // Rx
  kppJoint->kwsJoint()->dof(3)->isLocked(true);
  // Ry
  kppJoint->kwsJoint()->dof(4)->isLocked(true);

  //attach the body to the joint 

  // Create and attach hppBody to joint
  ChppBodyShPtr hppBody = ChppBody::create(polyBox->name());

  kppJoint->kwsJoint()->setAttachedBody( (CkwsBodyShPtr) hppBody);

  kppJoint->addSolidComponentRef( CkppSolidComponentRef::create(polyBox) ); 
  kppJoint->doesDisplayPath( true );

  //attach the joint to the device
  hppDevice->rootJointComponent(kppJoint);

  // add obstacles to polyBox
  // inner object with absPos  and outer object from obstacleList.

  // create collision list of body
  std::vector<CkcdObjectShPtr> collisionList ;
  // inner objects matrix list
  std::vector< CkitMat4 > matList;
  collisionList.clear();
  matList.clear();

  collisionList.push_back(polyBox);
  matList.push_back(absPos);

  hppBody->setInnerObjects(collisionList);
  hppBody->innerObjects(collisionList, matList);

  hppBody->setOuterObjects(obstacleList());
  setUserWeightScaler(kppJoint);
  return hppDevice ;
  
}

bool ChppWalkFootPlanner::setBoundingBoxSize(double inXmin, double inXmax, double inYmin, 
					     double inYmax, double inZmin, double inZmax)
{
  // Check validity of parameters
  if (inXmin >= inXmax) {
    ODEBUG1(":setBoundingBoxSize: inXmin >= inXmax: inXmin=" << inXmin << "inXmax=" << inXmax); 
    return false;
  }
  if (inYmin >= inYmax) {
    ODEBUG1(":setBoundingBoxSize: inYmin >= inYmax: inYmin=" << inYmin << "inYmax=" << inYmax); 
    return false;
  }
  if (inZmin >= inZmax) {
    ODEBUG1(":setBoundingBoxSize: inZmin >= inZmax: inZmin=" << inZmin << "inZmax=" << inZmax); 
    return false;
  }

  // Compute center and size of the box.
  double xCenter = .5*(inXmax+inXmin);
  double xLength = .5*(inXmax-inXmin);

  double yCenter = .5*(inYmax+inYmin);
  double yLength = .5*(inYmax-inYmin);

  double zCenter = .5*(inZmax+inZmin);
  double zLength = .5*(inZmax-inZmin);

  ODEBUG2(":setBoundingBoxSize: xCenter=" << xCenter);
  ODEBUG2(":setBoundingBoxSize: yCenter=" << xCenter);
  ODEBUG2(":setBoundingBoxSize: zCenter=" << xCenter);
  ODEBUG2(":setBoundingBoxSize: xLength=" << xLength);
  ODEBUG2(":setBoundingBoxSize: yLength=" << xLength);
  ODEBUG2(":setBoundingBoxSize: zLength=" << xLength);

  CkppKCDBoxShPtr box = getRobotBoundingBox();
  if (!box) {
    return false;
  }

  box->setSize(xLength, yLength, zLength);
  CkitMat4 relBoxPosition;
  relBoxPosition.translate(xCenter, yCenter, zCenter);
  box->setRelativePosition(relBoxPosition);

  CkitMat4 absolutePositionBox;
  CkitMat4 absolutePositionJoint;

  box->getAbsolutePosition(absolutePositionBox);

  ODEBUG2(":setBoundingBoxSize: absolute position of bounding box");
  ODEBUG2(":setBoundingBoxSize: " << absolutePositionBox);

  CkppDeviceComponentShPtr boxDevice = robotIthProblem(0);
  CkppJointComponentShPtr kppJoint = boxDevice->rootJointComponent();
  setUserWeightScaler(kppJoint);
  return true;
}


CkppKCDBoxShPtr ChppWalkFootPlanner::getRobotBoundingBox()
{
  CkppDeviceComponentShPtr boxDevice = robotIthProblem(0);
  if (!boxDevice) {
    ODEBUG1(":getRobotBoundingBox: cannot access first robot in ChppPlanner object");
    return CkppKCDBoxShPtr();
  }

  // Get vector of solid components
  std::vector< CkppSolidComponentRefShPtr > solidComponentRefVector;
  boxDevice->getSolidComponentRefVector(solidComponentRefVector);

  unsigned int numberSolids = solidComponentRefVector.size();
  if (numberSolids != 1) {
    ODEBUG1(":getRobotBoundingBox: number of solid objects is " << numberSolids << " != 1");
    ODEBUG1(":getRobotBoundingBox: taking the first one");
  }
  
  CkppSolidComponentRefShPtr solidComponentRef = solidComponentRefVector[0];
  CkppSolidComponentShPtr solidComponent = solidComponentRef->referencedSolidComponent();
  
  // Test that solid object is a box
  CkppKCDBoxShPtr box;
  if (!(box = KIT_DYNAMIC_PTR_CAST(CkppKCDBox, solidComponent))) {
    ODEBUG1(":getRobotBoundingBox: solid component is not a box");
    return CkppKCDBoxShPtr();
  }
  return box;
}


void ChppWalkFootPlanner::setUserWeightScaler(CkppJointComponentShPtr inKppJoint)
{  
  /*
  Set userWeightScaler
  */
  inKppJoint->kwsJoint()->dof(0)->userWeightScaler(1.5);
  inKppJoint->kwsJoint()->dof(1)->userWeightScaler(1.5);
  double weight = inKppJoint->kwsJoint()->dof(5)->weight();
  if (weight > 1e-6) {
    inKppJoint->kwsJoint()->dof(5)->userWeightScaler(6.0*0.5*stepSpacing()/weight);
  }
}
