/*
  Copyright 2008 CNRS
  Author: Florent Lamiraux
*/

#include <iostream>

#include "KineoWorks2/kwsConfig.h"
#include "KineoWorks2/kwsDevice.h"
#include "KineoWorks2/kwsJoint.h"
#include "KineoWorks2/kwsJointFreeFlyer.h"
#include "KineoWorks2/kwsDof.h"
#include "KineoWorks2/kwsJointDof.h"
#include "KineoWorks2/kwsValidator.h"
#include "KineoWorks2/kwsValidatorDPCollision.h"
#include "KineoWorks2/kwsNode.h"
#include "KineoWorks2/kwsEdge.h"
#include "KineoWorks2/kwsRoadmap.h"
#include "KineoWorks2/kwsRoadmapBuilder.h"
#include "KineoWorks2/kwsDiffusingRdmBuilder.h"
#include "KineoWorks2/kwsSteeringMethod.h"

#include "hppWalkFootPlanner/hppWalkPlannerDelegate.h"

// Select lecel of verbosity at configuration.
#if DEBUG==2
#define ODEBUG2(x) std::cout << "ChppWalkPlannerDelegate:" << x << std::endl
#define ODEBUG1(x) std::cerr << "ChppWalkPlannerDelegate:" << x << std::endl
#elif DEBUG==1
#define ODEBUG2(x)
#define ODEBUG1(x) std::cerr << "ChppWalkPlannerDelegate:" << x << std::endl
#else
#define ODEBUG2(x)
#define ODEBUG1(x)
#endif


void ChppWalkPlannerDelegate::builderDidAddNode(const CkwsRoadmapBuilderConstShPtr &inBuilder, 
						const CkwsNodeConstShPtr &inNode)
{
  ODEBUG2(" entering method builderDidAddNode()");
  CkwsRdmBuilderDelegate::builderDidAddNode(inBuilder, inNode);
  if (nodeAddedByThisDelegate) {
    nodeAddedByThisDelegate = false;
  }
  else {
    CkwsRoadmapBuilderShPtr roadmapBuilder = attRdmBuilder.lock();
    double penetration = roadmapBuilder->penetration();
    CkwsConfig config1(inNode->config());
    CkwsConfig config2(config1);
    CkwsDeviceShPtr device = config1.device();
    CkwsJointShPtr rootJoint = device->rootJoint();
    
    /*
      Copy set of direct path validators of the device
    */
    CkwsDirectPathValidatorSetConstShPtr dpValidator =
      CkwsDirectPathValidatorSet::createCopy(device->directPathValidators());;

    /*
      Retrieve collision direct path validator and set penetration
    */
    CkwsValidatorDPCollisionShPtr collisionValidator = 
      dpValidator->retrieve<CkwsValidatorDPCollision>();
    if (collisionValidator) {
      collisionValidator->penetration(penetration);
    }

    if (KIT_DYNAMIC_PTR_CAST(CkwsJointFreeFlyer, rootJoint)) {
      CkwsDofShPtr Rz = rootJoint->dof(5);
      unsigned int dofRank;
      device->getRankOfDof(Rz, dofRank);
      config2.dofValue(dofRank, config1.dofValue(dofRank)+3.1415); // slightly less than pi.

      /*
	Once the counterpart config has been created, validate the config
      */
      CkwsConfigValidatorSetConstShPtr configValidatorSet=device->configValidators();
      configValidatorSet->validate(config2);
      if (config2.isValid()) {
	/*
	  Test whether node is already in the roadmap
	*/
	CkwsRoadmapShPtr roadmap = roadmapBuilder->roadmap();
	if (roadmap) {
	  CkwsNodeShPtr node1 = roadmap->nodeWithConfig(config1);
	  CkwsNodeShPtr node2 = roadmap->nodeWithConfig(config2);
	  if (!node2) {
	    node2 = CkwsNode::create(config2);
	    nodeAddedByThisDelegate = true;
	    if (roadmapBuilder->addNode(node2) == KD_ERROR) {
	      nodeAddedByThisDelegate = false;
	      ODEBUG2(":builderDidAddNode: failed to add a node with valid config");
	      return;
	    }
	    
	    ODEBUG2(":builderDidAddNode: successfully added a node.");
	  }
	  /*
	    Try to insert edge
	  */
	  CkwsSteeringMethodShPtr sm = attSteeringMethod;
	  if (!sm) {
	    sm = device->steeringMethod();
	  }
	  if (sm) {
	    CkwsDirectPathShPtr dp = sm->makeDirectPath(config1, config2);
	    if (dp) {
	      /*
		Validate direct path
	      */
	      dpValidator->validate(*dp);
	      
	      if (dp->isValid()) {
		
		CkwsEdgeShPtr edge=CkwsEdge::create(dp);
		
		edgeAddedByThisDelegate = true;
		if (roadmapBuilder->addEdge(node1, node2, edge) == KD_ERROR) {
		  ODEBUG2(":builderDidAddNode: failed to add edge in roadmap: config1->config2");
		}
		else {
		  ODEBUG2(":builderDidAddNode: successfully added an edge.");
		}
		edgeAddedByThisDelegate = false;
	      }
	    }
	    dp =  sm->makeDirectPath(config2, config1);
	    if (dp) {
	      /*
		Validate direct path
	      */
	      dpValidator->validate(*dp);
	      
	      if (dp->isValid()) {
		
		CkwsEdgeShPtr edge=CkwsEdge::create(dp);
		edgeAddedByThisDelegate = true;
		if (roadmapBuilder->addEdge(node2, node1, edge) == KD_ERROR) {
		  ODEBUG2(":builderDidAddNode: failed to add edge in roadmap: config2->config1");
		}
		else {
		  ODEBUG2(":builderDidAddNode: successfully added an edge.");
		}
		edgeAddedByThisDelegate = false;
	      }
	    }
	  }
	  else {
	    ODEBUG2(":builderDidAddNode: no steering method in delegate nor in device");
	  }
	}
	else {
	  ODEBUG2(":builderDidAddNode: no roadmap in roadmap builder.");
	}
      }
      else {
	ODEBUG2(":builderDidAddNode: config not valid.");
      }
    }
  }
  nodeAddedByThisDelegate = false;
  edgeAddedByThisDelegate = false;
}

void ChppWalkPlannerDelegate::builderDidAddEdge(const CkwsRoadmapBuilderConstShPtr &inBuilder, 
						const CkwsEdgeConstShPtr &inEdge)
{
  CkwsRdmBuilderDelegate::builderDidAddEdge(inBuilder, inEdge);
  if (edgeAddedByThisDelegate) {
    edgeAddedByThisDelegate = false;
  }
  else {
  }
}

